==> builders_inventory-2.1.4-sources.jar: _3650/builders_inventory/mixin/feature/minimessage/screens/chat/ChatScreenMixin.java <==
package _3650.builders_inventory.mixin.feature.minimessage.screens.chat;

import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2583;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_338;
import net.minecraft.class_342;
import net.minecraft.class_3532;
import net.minecraft.class_408;
import net.minecraft.class_437;
import net.minecraft.class_4717;
import net.minecraft.class_8666;
import org.lwjgl.glfw.GLFW;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.ModifyArg;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;

import _3650.builders_inventory.BuildersInventory;
import _3650.builders_inventory.api.minimessage.MiniMessageUtil;
import _3650.builders_inventory.api.minimessage.instance.LastParseListener;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance.PreviewOptions;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance.SuggestionOptions;
import _3650.builders_inventory.api.minimessage.widgets.wrapper.WrappedTextField;
import _3650.builders_inventory.api.widgets.exbutton.ExtendedImageButtonGui;
import _3650.builders_inventory.api.widgets.exbutton.ExtendedImageDualButton;
import _3650.builders_inventory.config.Config;
import _3650.builders_inventory.feature.minimessage.chat.ChatMiniMessageContext;
import _3650.builders_inventory.feature.minimessage.chat.ChatMiniMessageButtonDisplay;

@Mixin(class_408.class)
public abstract class ChatScreenMixin extends ScreenMixinOverrides {
	
	@Unique
	private static final class_8666 SPRITES_BUTTON_FORCE = new class_8666(
			BuildersInventory.modLoc("minimessage/button_force"),
			BuildersInventory.modLoc("minimessage/button_force_highlighted"));
	@Unique
	private static final class_8666 SPRITES_BUTTON_FORCE_ACTIVE = new class_8666(
			BuildersInventory.modLoc("minimessage/button_force_active"),
			BuildersInventory.modLoc("minimessage/button_force_active_highlighted"));
	
	@Shadow
	protected class_342 input;
	@Shadow
	private String initial;
	@Shadow
	class_4717 commandSuggestions;
	
	@Unique
	private ExtendedImageButtonGui exGui = new ExtendedImageButtonGui();
	@Unique
	private ExtendedImageDualButton buttonForce;
	@Unique
	private boolean resetFocus = false;
	@Unique
	private MiniMessageInstance minimessage;
	
	@Inject(method = "init", at = @At("TAIL"), order = Integer.MAX_VALUE - 1)
	private void builders_inventory_initChat(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		// load tagWidget
		this.minimessage = new MiniMessageInstance(
				this.minecraft,
				(class_437)(Object)this,
				this.font,
				WrappedTextField.editBox(this.input),
				ChatMiniMessageContext::isValid,
				LastParseListener.IGNORE,
				PreviewOptions.chat(),
				SuggestionOptions.chat(() -> this.commandSuggestions)
				);
		
		// override formatter
		MiniMessageUtil.wrapFormatter(this.input, this.minimessage);
		
		// exGui for button
		this.exGui.init();
		
		// force minimessage button
		final var buttonDisplay = Config.instance().minimessage_chatForceButtonDisplay;
		if (buttonDisplay != ChatMiniMessageButtonDisplay.NONE) {
			// calculate positions (calculations are order-independent)
			final var inputWidth = buttonDisplay.chatWidth(this.input.method_25368());
			final var inputX = buttonDisplay.chatX(this.input.method_46426());
			final int buttonX = buttonDisplay.buttonX(this.input.method_46426(), this.input.method_25368());
			
			// resize/move input to make room for button
			this.input.method_25358(inputWidth);
			this.input.method_46421(inputX);
			
			// make buttons
			this.buttonForce = new ExtendedImageDualButton(buttonX, this.height - 13, 10, 10,
					SPRITES_BUTTON_FORCE,
					button -> {
						if (class_437.method_25442() && class_437.method_25441()) ChatMiniMessageContext.loadServerCommandMap();
						else this.updateForce(true);
						this.resetFocus = true;
					},
					List.of(
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force").method_27692(class_124.field_1068),
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force.desc.enable").method_27692(class_124.field_1080)),
					SPRITES_BUTTON_FORCE_ACTIVE,
					button -> {
						if (class_437.method_25442() && class_437.method_25441()) ChatMiniMessageContext.loadServerCommandMap();
						else this.updateForce(false);
						this.resetFocus = true;
					},
					List.of(
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force").method_27692(class_124.field_1068),
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force.desc.disable").method_27692(class_124.field_1080))
					);
			
			this.exGui.addRenderableWidget(buttonForce);
			this.addRenderableWidget(buttonForce);
			
			this.updateForce(ChatMiniMessageContext.forceChatMinimessage);
		}
	}
	
	@Inject(method = "resize", at = @At("TAIL"))
	private void builders_inventory_resizeChat(class_310 mc, int width, int height, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.lastParse != null) this.commandSuggestions.method_23933(false);
		this.minimessage.reposition();
	}
	
	@Inject(method = "onEdited", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_editChat(String value, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.unknownEdit()) ci.cancel();
	}
	
	@Inject(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/Screen;render(Lnet/minecraft/client/gui/GuiGraphics;IIF)V"))
	private void builders_inventory_renderPreviewAndError(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.minimessage.renderPreviewOrError(gui);
	}
	
	@WrapOperation(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/CommandSuggestions;render(Lnet/minecraft/client/gui/GuiGraphics;II)V"))
	private void builders_inventory_renderSuggestions(class_4717 commandSuggestions, class_332 gui, int mouseX, int mouseY, Operation<Void> operation) {
		if (!Config.instance().minimessage_enabledChat) {
			operation.call(commandSuggestions, gui, mouseX, mouseY);
			return;
		}
		if (this.minimessage.previewLines.isEmpty()) operation.call(commandSuggestions, gui, mouseX, mouseY);
		else this.minimessage.renderSuggestions(gui, mouseX, mouseY);
	}
	
	@Inject(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;getMessageTagAt(DD)Lnet/minecraft/client/GuiMessageTag;"), cancellable = true)
	private void builders_inventory_renderHover(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.renderHover(gui, mouseX, mouseY)) ci.cancel();
		else if (this.minimessage.renderFormatHover(gui, mouseX, mouseY, 0, this.input.method_1882().length())) ci.cancel();
		else if (this.exGui.renderTooltip(font, gui, mouseX, mouseY)) ci.cancel();
	}
	
	@Inject(method = "keyPressed", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_keyPressed(int keyCode, int scanCode, int modifiers, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.keyPressed(keyCode, scanCode, modifiers)) cir.setReturnValue(true);
		else if (keyCode == GLFW.GLFW_KEY_TAB && !this.minimessage.previewLines.isEmpty()) cir.setReturnValue(true);
	}
	
	@Inject(method = "mouseScrolled", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_mouseScrolled(double mouseX, double mouseY, double scrollX, double scrollY, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.mouseScrolled(mouseX, mouseY, scrollX, scrollY)) cir.setReturnValue(true);
	}
	
	@Inject(method = "mouseClicked", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_mouseClicked(double mouseX, double mouseY, int button, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.mouseClicked(mouseX, mouseY, button)) cir.setReturnValue(true);
	}
	
	@WrapOperation(method = "mouseClicked", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/ChatScreen;getComponentStyleAt(DD)Lnet/minecraft/network/chat/Style;"))
	private class_2583 builders_inventory_previewClicked(class_408 screen, double mouseX, double mouseY, Operation<class_2583> operation) {
		if (!Config.instance().minimessage_enabledChat) {
			return operation.call(screen, mouseX, mouseY);
		}
		class_2583 style = this.minimessage.tryClickPreview(mouseX, mouseY);
		if (style != null) return style;
		else return operation.call(screen, mouseX, mouseY);
	}
	
	@WrapOperation(method = "moveInHistory", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/EditBox;setValue(Ljava/lang/String;)V"))
	private void builders_inventory_historyMove(class_342 input, String value, Operation<Void> operation) {
		if (!Config.instance().minimessage_enabledChat) {
			operation.call(input, value);
			return;
		}
		this.minimessage.suppressSuggestionUpdate = true;
		operation.call(input, value);
		this.minimessage.suppressSuggestionUpdate = false;
	}
	
	@Override
	protected void tickInjectTail(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.minimessage.tick();
	}
	
	/*
	 * Button stuff
	 */
	
	@Override
	protected void clearWidgetsInjectTail(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.exGui.clearWidgets();
	}
	
	@Unique
	private void updateForce(boolean force) {
		ChatMiniMessageContext.forceChatMinimessage = force;
		if (force) {
			this.buttonForce.secondMode = true;
			this.minimessage.unknownEdit();
		} else {
			this.buttonForce.secondMode = false;
			this.minimessage.clearParse();
		}
	}
	
	@Inject(method = "render", at = @At("HEAD"))
	private void builders_inventory_resetFocus(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (this.resetFocus) {
			this.resetFocus = false;
			this.method_25395(this.input);
		}
	}
	
	/*
	 * Moving chat if preview is too tall
	 */
	
	@WrapOperation(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;render(Lnet/minecraft/client/gui/GuiGraphics;IIIZ)V"))
	private void builders_inventory_moveChatComponent(class_338 chat, class_332 gui, int tickCount, int mouseX, int mouseY, boolean focused, Operation<Void> operation) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			gui.method_51448().pushMatrix();
			gui.method_51448().translate(0, -class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold)));
			operation.call(chat, gui, tickCount, mouseX, mouseY, focused);
			gui.method_51448().popMatrix();
		} else {
			operation.call(chat, gui, tickCount, mouseX, mouseY, focused);
		}
	}
	
	@ModifyArg(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;getMessageTagAt(DD)Lnet/minecraft/client/GuiMessageTag;"), index = 1)
	private double builders_inventory_moveLabelMouseY(double mouseY) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			return mouseY + class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold));
		} else return mouseY;
	}
	
	@ModifyArg(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/ChatScreen;getComponentStyleAt(DD)Lnet/minecraft/network/chat/Style;"), index = 1)
	private double builders_inventory_moveHoverMouseY(double mouseY) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			return mouseY + class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold));
		} else return mouseY;
	}
	
}

==> builders_inventory-2.2.0-sources.jar: _3650/builders_inventory/mixin/feature/minimessage/screens/chat/ChatScreenMixin.java <==
package _3650.builders_inventory.mixin.feature.minimessage.screens.chat;

import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2583;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_338;
import net.minecraft.class_342;
import net.minecraft.class_3532;
import net.minecraft.class_408;
import net.minecraft.class_437;
import net.minecraft.class_4717;
import net.minecraft.class_8666;
import org.lwjgl.glfw.GLFW;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.ModifyArg;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;

import _3650.builders_inventory.BuildersInventory;
import _3650.builders_inventory.api.minimessage.MiniMessageUtil;
import _3650.builders_inventory.api.minimessage.instance.LastParseListener;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance.PreviewOptions;
import _3650.builders_inventory.api.minimessage.instance.MiniMessageInstance.SuggestionOptions;
import _3650.builders_inventory.api.minimessage.widgets.wrapper.WrappedTextField;
import _3650.builders_inventory.api.widgets.exbutton.ExtendedImageButtonGui;
import _3650.builders_inventory.api.widgets.exbutton.ExtendedImageDualButton;
import _3650.builders_inventory.config.Config;
import _3650.builders_inventory.feature.minimessage.chat.ChatMiniMessageContext;
import _3650.builders_inventory.feature.minimessage.chat.ChatMiniMessageButtonDisplay;

@Mixin(class_408.class)
public abstract class ChatScreenMixin extends ScreenMixinOverrides {
	
	@Unique
	private static final class_8666 SPRITES_BUTTON_FORCE = new class_8666(
			BuildersInventory.modLoc("minimessage/button_force"),
			BuildersInventory.modLoc("minimessage/button_force_highlighted"));
	@Unique
	private static final class_8666 SPRITES_BUTTON_FORCE_ACTIVE = new class_8666(
			BuildersInventory.modLoc("minimessage/button_force_active"),
			BuildersInventory.modLoc("minimessage/button_force_active_highlighted"));
	
	@Shadow
	protected class_342 input;
	@Shadow
	private String initial;
	@Shadow
	class_4717 commandSuggestions;
	
	@Unique
	private ExtendedImageButtonGui exGui = new ExtendedImageButtonGui();
	@Unique
	private ExtendedImageDualButton buttonForce;
	@Unique
	private boolean resetFocus = false;
	@Unique
	private MiniMessageInstance minimessage;
	
	@Inject(method = "init", at = @At("TAIL"), order = Integer.MAX_VALUE - 1)
	private void builders_inventory_initChat(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		// load tagWidget
		this.minimessage = new MiniMessageInstance(
				this.minecraft,
				(class_437)(Object)this,
				this.font,
				WrappedTextField.editBox(this.input),
				ChatMiniMessageContext::isValid,
				LastParseListener.IGNORE,
				PreviewOptions.chat(),
				SuggestionOptions.chat(() -> this.commandSuggestions)
				);
		
		// override formatter
		MiniMessageUtil.wrapFormatter(this.input, this.minimessage);
		
		// exGui for button
		this.exGui.init();
		
		// force minimessage button
		final var buttonDisplay = Config.instance().minimessage_chatForceButtonDisplay;
		if (buttonDisplay != ChatMiniMessageButtonDisplay.NONE) {
			// calculate positions (calculations are order-independent)
			final var inputWidth = buttonDisplay.chatWidth(this.input.method_25368());
			final var inputX = buttonDisplay.chatX(this.input.method_46426());
			final int buttonX = buttonDisplay.buttonX(this.input.method_46426(), this.input.method_25368());
			
			// resize/move input to make room for button
			this.input.method_25358(inputWidth);
			this.input.method_46421(inputX);
			
			// make buttons
			this.buttonForce = new ExtendedImageDualButton(buttonX, this.height - 13, 10, 10,
					SPRITES_BUTTON_FORCE,
					button -> {
						if (class_437.method_25442() && class_437.method_25441()) ChatMiniMessageContext.loadServerCommandMap();
						else this.updateForce(true);
						this.resetFocus = true;
					},
					List.of(
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force").method_27692(class_124.field_1068),
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force.desc.enable").method_27692(class_124.field_1080)),
					SPRITES_BUTTON_FORCE_ACTIVE,
					button -> {
						if (class_437.method_25442() && class_437.method_25441()) ChatMiniMessageContext.loadServerCommandMap();
						else this.updateForce(false);
						this.resetFocus = true;
					},
					List.of(
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force").method_27692(class_124.field_1068),
							class_2561.method_43471("container.builders_inventory.minimessage.tooltip.button.chat_force.desc.disable").method_27692(class_124.field_1080))
					);
			
			this.exGui.addRenderableWidget(buttonForce);
			this.addRenderableWidget(buttonForce);
			
			this.updateForce(ChatMiniMessageContext.forceChatMinimessage);
		}
	}
	
	@Inject(method = "resize", at = @At("TAIL"))
	private void builders_inventory_resizeChat(class_310 mc, int width, int height, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.lastParse != null) this.commandSuggestions.method_23933(false);
		this.minimessage.reposition();
	}
	
	@Inject(method = "onEdited", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_editChat(String value, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.unknownEdit()) ci.cancel();
	}
	
	@Inject(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/Screen;render(Lnet/minecraft/client/gui/GuiGraphics;IIF)V"))
	private void builders_inventory_renderPreviewAndError(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.minimessage.renderPreviewOrError(gui);
	}
	
	@WrapOperation(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/CommandSuggestions;render(Lnet/minecraft/client/gui/GuiGraphics;II)V"))
	private void builders_inventory_renderSuggestions(class_4717 commandSuggestions, class_332 gui, int mouseX, int mouseY, Operation<Void> operation) {
		if (!Config.instance().minimessage_enabledChat || this.minimessage.previewLines.isEmpty()) {
			operation.call(commandSuggestions, gui, mouseX, mouseY);
		} else {
			this.minimessage.renderSuggestions(gui, mouseX, mouseY);
		}
	}
	
	@Inject(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;getMessageTagAt(DD)Lnet/minecraft/client/GuiMessageTag;"), cancellable = true)
	private void builders_inventory_renderHover(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.renderHover(gui, mouseX, mouseY)) ci.cancel();
		else if (this.minimessage.renderFormatHover(gui, mouseX, mouseY, 0, this.input.method_1882().length())) ci.cancel();
		else if (this.exGui.renderTooltip(font, gui, mouseX, mouseY)) ci.cancel();
	}
	
	@Inject(method = "keyPressed", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_keyPressed(int keyCode, int scanCode, int modifiers, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.keyPressed(keyCode, scanCode, modifiers)) cir.setReturnValue(true);
		else if (keyCode == GLFW.GLFW_KEY_TAB && !this.minimessage.previewLines.isEmpty()) cir.setReturnValue(true);
	}
	
	@Inject(method = "mouseScrolled", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_mouseScrolled(double mouseX, double mouseY, double scrollX, double scrollY, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.mouseScrolled(mouseX, mouseY, scrollX, scrollY)) cir.setReturnValue(true);
	}
	
	@Inject(method = "mouseClicked", at = @At("HEAD"), cancellable = true)
	private void builders_inventory_mouseClicked(double mouseX, double mouseY, int button, CallbackInfoReturnable<Boolean> cir) {
		if (!Config.instance().minimessage_enabledChat) return;
		if (this.minimessage.mouseClicked(mouseX, mouseY, button)) cir.setReturnValue(true);
	}
	
	@WrapOperation(method = "mouseClicked", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/ChatScreen;getComponentStyleAt(DD)Lnet/minecraft/network/chat/Style;"))
	private class_2583 builders_inventory_previewClicked(class_408 screen, double mouseX, double mouseY, Operation<class_2583> operation) {
		if (!Config.instance().minimessage_enabledChat) {
			return operation.call(screen, mouseX, mouseY);
		}
		class_2583 style = this.minimessage.tryClickPreview(mouseX, mouseY);
		if (style != null) return style;
		else return operation.call(screen, mouseX, mouseY);
	}
	
	@WrapOperation(method = "moveInHistory", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/EditBox;setValue(Ljava/lang/String;)V"))
	private void builders_inventory_historyMove(class_342 input, String value, Operation<Void> operation) {
		if (Config.instance().minimessage_enabledChat) {
			this.minimessage.suppressSuggestionUpdate = true;
			operation.call(input, value);
			this.minimessage.suppressSuggestionUpdate = false;
		} else {
			operation.call(input, value);
		}
	}
	
	@Override
	protected void tickInjectTail(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.minimessage.tick();
	}
	
	/*
	 * Button stuff
	 */
	
	@Override
	protected void clearWidgetsInjectTail(CallbackInfo ci) {
		if (!Config.instance().minimessage_enabledChat) return;
		this.exGui.clearWidgets();
	}
	
	@Unique
	private void updateForce(boolean force) {
		ChatMiniMessageContext.forceChatMinimessage = force;
		if (force) {
			this.buttonForce.secondMode = true;
			this.minimessage.unknownEdit();
		} else {
			this.buttonForce.secondMode = false;
			this.minimessage.clearParse();
		}
	}
	
	@Inject(method = "render", at = @At("HEAD"))
	private void builders_inventory_resetFocus(class_332 gui, int mouseX, int mouseY, float partialTick, CallbackInfo ci) {
		if (this.resetFocus) {
			this.resetFocus = false;
			this.method_25395(this.input);
		}
	}
	
	/*
	 * Moving chat if preview is too tall
	 */
	
	@WrapOperation(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;render(Lnet/minecraft/client/gui/GuiGraphics;IIIZ)V"))
	private void builders_inventory_moveChatComponent(class_338 chat, class_332 gui, int tickCount, int mouseX, int mouseY, boolean focused, Operation<Void> operation) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			gui.method_51448().method_22903();
			gui.method_51448().method_46416(0, -class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold)), 0);
			operation.call(chat, gui, tickCount, mouseX, mouseY, focused);
			gui.method_51448().method_22909();
		} else {
			operation.call(chat, gui, tickCount, mouseX, mouseY, focused);
		}
	}
	
	@ModifyArg(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/components/ChatComponent;getMessageTagAt(DD)Lnet/minecraft/client/GuiMessageTag;"), index = 1)
	private double builders_inventory_moveLabelMouseY(double mouseY) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			return mouseY + class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold));
		} else return mouseY;
	}
	
	@ModifyArg(method = "render", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/gui/screens/ChatScreen;getComponentStyleAt(DD)Lnet/minecraft/network/chat/Style;"), index = 1)
	private double builders_inventory_moveHoverMouseY(double mouseY) {
		final Config config = Config.instance();
		final int offsetThreshold = config.minimessage_previewOffsetIgnored;
		if (config.minimessage_enabledChat && config.minimessage_previewOffsetsChat && this.minimessage.previewLines.size() > offsetThreshold) {
			return mouseY + class_3532.method_15386(this.minimessage.getScaledLineHeight(offsetThreshold));
		} else return mouseY;
	}
	
}
